@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004-2021 musikcube team
+// Copyright (c) 2004-2023 musikcube team
 //
 // All rights reserved.
 //
@@ -39,17 +39,16 @@
 #include "TranscodingAudioDataStream.h"
 
 #include <musikcore/sdk/ITrack.h>
+#include <musikcore/sdk/String.h>
 
 #pragma warning(push, 0)
-#include <boost/filesystem.hpp>
-#include <boost/algorithm/string.hpp>
 #include <websocketpp/base64/base64.hpp>
 #pragma warning(pop, 0)
 
-#include <iostream>
 #include <unordered_map>
 #include <string>
 #include <cstdlib>
+#include <filesystem>
 
 #include <fcntl.h>
 #include <stdio.h>
@@ -61,8 +60,14 @@
 #include <vector>
 
 #define HTTP_416_DISABLED true
+#define ENABLE_DEBUG 1
 
 static const char* ENVIRONMENT_DISABLE_HTTP_SERVER_AUTH = "MUSIKCUBE_DISABLE_HTTP_SERVER_AUTH";
+static const char* TAG = "HttpServer";
+
+namespace std {
+    namespace fs = std::filesystem;
+}
 
 using namespace musik::core::sdk;
 
@@ -98,10 +103,8 @@ struct Range {
 
 static std::string contentType(const std::string& fn) {
     try {
-        boost::filesystem::path p(fn);
-        std::string ext = boost::trim_copy(p.extension().string());
-        boost::to_lower(ext);
-
+        std::fs::path p(std::fs::u8path(fn));
+        std::string ext = str::ToLowerCopy(str::Trim(p.extension().u8string()));
         auto it = CONTENT_TYPE_MAP.find(ext);
         if (it != CONTENT_TYPE_MAP.end()) {
             return it->second;
@@ -115,10 +118,9 @@ static std::string contentType(const std::string& fn) {
 
 static std::string fileExtension(const std::string& fn) {
     try {
-        boost::filesystem::path p(fn);
-        std::string ext = boost::trim_copy(p.extension().string());
+        std::fs::path p(std::fs::u8path(fn));
+        std::string ext = str::ToLowerCopy(str::Trim(p.extension().u8string()));
         if (ext.size()) {
-            boost::to_lower(ext);
             return ext[0] == '.' ? ext.substr(1) : ext;
         }
     }
@@ -154,10 +156,6 @@ static ssize_t fileReadCallback(void *cls, uint64_t pos, char *buf, size_t max)
 static void fileFreeCallback(void *cls) {
     Range* range = static_cast<Range*>(cls);
     if (range->file) {
-#ifdef ENABLE_DEBUG
-        std::cerr << "******** REQUEST CLOSE: " << range->file << " ********\n\n";
-#endif
-
         range->file->Close(); /* lazy destroy */
         range->file = nullptr;
     }
@@ -179,17 +177,14 @@ static Range* parseRange(IDataStream* file, const char* range) {
 
         if (str.substr(0, 6) == "bytes=") {
             str = str.substr(6);
-
-            std::vector<std::string> parts;
-            boost::split(parts, str, boost::is_any_of("-"));
-
+            std::vector<std::string> parts = str::Split(str, "-");
             if (parts.size() == 2) {
                 try {
-                    size_t from = (size_t) std::max(0, std::stoi(boost::algorithm::trim_copy(parts[0])));
+                    size_t from = (size_t) std::max(0, std::stoi(str::Trim(parts[0])));
                     size_t to = size;
 
                     if (parts.at(1).size()) {
-                        to = (size_t) std::min((int) size, std::stoi(boost::algorithm::trim_copy(parts[1])));
+                        to = (size_t) std::min((int) size, std::stoi(str::Trim(parts[1])));
                     }
 
                     if (to > from) {
@@ -267,10 +262,7 @@ static bool isAuthenticated(MHD_Connection *connection, Context& context) {
             std::string encoded = auth.substr(6);
             if (encoded.size()) {
                 std::string decoded = websocketpp::base64_decode(encoded);
-
-                std::vector<std::string> userPass;
-                boost::split(userPass, decoded, boost::is_any_of(":"));
-
+                std::vector<std::string> userPass = str::Split(decoded, ":");
                 if (userPass.size() == 2) {
                     std::string password = GetPreferenceString(context.prefs, key::password, defaults::password);
                     return userPass[0] == "default" && userPass[1] == password;
@@ -367,11 +359,11 @@ MHD_Result HttpServer::HandleRequest(
     size_t *upload_data_size,
     void **con_cls)
 {
-#ifdef ENABLE_DEBUG
-    std::cerr << "******** REQUEST START ********\n";
-#endif
+    auto server = static_cast<HttpServer*>(cls);
 
-    HttpServer* server = static_cast<HttpServer*>(cls);
+#ifdef ENABLE_DEBUG
+    server->context.debug->Info(TAG, str::Format("new request: %s", url).c_str());
+#endif
 
     struct MHD_Response* response = nullptr;
     int ret = MHD_NO;
@@ -383,9 +375,8 @@ MHD_Result HttpServer::HandleRequest(
                 status = 401; /* unauthorized */
                 static const char* error = "unauthorized";
                 response = MHD_create_response_from_buffer(strlen(error), (void*)error, MHD_RESPMEM_PERSISTENT);
-
 #ifdef ENABLE_DEBUG
-                std::cerr << "unauthorized\n";
+                server->context.debug->Warning(TAG, "unauthorized request, returning a 401");
 #endif
             }
             else {
@@ -396,8 +387,7 @@ MHD_Result HttpServer::HandleRequest(
                     urlStr = urlStr.substr(1);
                 }
 
-                std::vector<std::string> parts;
-                boost::split(parts, urlStr, boost::is_any_of("/"));
+                std::vector<std::string> parts = str::Split(urlStr, "/");
                 if (parts.size() > 0) {
                     /* /audio/id/<id> OR /audio/external_id/<external_id> */
                     if (parts.at(0) == fragment::audio && parts.size() == 3) {
@@ -416,17 +406,13 @@ MHD_Result HttpServer::HandleRequest(
 
     if (response) {
 #ifdef ENABLE_DEBUG
-        std::cerr << "returning with http code: " << status << std::endl;
+        server->context.debug->Info(TAG, str::Format("return http %d", status).c_str());
 #endif
 
         ret = MHD_queue_response(connection, status, response);
         MHD_destroy_response(response);
     }
 
-#ifdef ENABLE_DEBUG
-    std::cerr << "*******************************\n\n";
-#endif
-
     return (MHD_Result) ret;
 }
 
@@ -454,11 +440,6 @@ int HttpServer::HandleAudioTrackRequest(
     if (byExternalId) {
         std::string externalId = urlDecode(pathParts.at(2));
         track = server->context.metadataProxy->QueryTrackByExternalId(externalId.c_str());
-
-#ifdef ENABLE_DEBUG
-        std::cerr << "externalId: " << externalId << "\n";
-        std::cerr << "title: " << GetMetadataString(track, "title") << std::endl;
-#endif
     }
     else if (pathParts.at(1) == fragment::id) {
         uint64_t id = std::stoull(urlDecode(pathParts.at(2)));
@@ -473,7 +454,7 @@ int HttpServer::HandleAudioTrackRequest(
 
         track->Release();
 
-        std::string format = "";
+        std::string format;
 
         if (bitrate != 0) {
             format = getStringUrlParam(connection, "format", "mp3");
@@ -486,23 +467,17 @@ int HttpServer::HandleAudioTrackRequest(
         const char* rangeVal = MHD_lookup_connection_value(
             connection, MHD_HEADER_KIND, "Range");
 
-#ifdef ENABLE_DEBUG
-        if (rangeVal) {
-            std::cerr << "range header: " << rangeVal << "\n";
-        }
-#endif
-
         Range* range = parseRange(file, rangeVal);
 
-#ifdef ENABLE_DEBUG
-        std::cerr << "potential response header : " << range->HeaderValue() << std::endl;
-#endif
-
         /* ehh... */
         bool isOnDemandTranscoder = !!dynamic_cast<TranscodingAudioDataStream*>(file);
 
 #ifdef ENABLE_DEBUG
-        std::cerr << "on demand? " << isOnDemandTranscoder << std::endl;
+        server->context.debug->Info(TAG, str::Format(
+            "range request: %s, resolved range: %s, isOnDemandTranscoder=%s",
+            rangeVal ? rangeVal : "[unspecified]",
+            range ? range->HeaderValue().c_str() : "[unresolved]",
+            isOnDemandTranscoder ? "true" : "false").c_str());
 #endif
 
         /* gotta be careful with request ranges if we're transcoding. don't
@@ -512,7 +487,7 @@ int HttpServer::HandleAudioTrackRequest(
                 delete range;
 
 #ifdef ENABLE_DEBUG
-                std::cerr << "removing range header, seek requested with ondemand transcoder\n";
+                server->context.debug->Info(TAG, "removing range header, seek requested with ondemand transcoder");
 #endif
 
                 if (HTTP_416_DISABLED) {
@@ -561,8 +536,7 @@ int HttpServer::HandleAudioTrackRequest(
                 &fileFreeCallback);
 
 #ifdef ENABLE_DEBUG
-            std::cerr << "response length: " << ((length == 0) ? 0 : length + 1) << "\n";
-            std::cerr << "id: " << file << "\n";
+            server->context.debug->Info(TAG, str::Format("response length=%d", ((length == 0) ? 0 : length + 1)).c_str());
 #endif
 
             if (response) {
@@ -574,7 +548,7 @@ int HttpServer::HandleAudioTrackRequest(
                 if (!isOnDemandTranscoder) {
                     MHD_add_response_header(response, "Accept-Ranges", "bytes");
 
-                    if (boost::filesystem::exists(title)) {
+                    if (std::fs::exists(std::fs::u8path(filename))) {
                         MHD_add_response_header(response, "X-musikcube-Filename-Override", externalId.c_str());
                     }
                 }
@@ -606,7 +580,7 @@ int HttpServer::HandleAudioTrackRequest(
                         status = MHD_HTTP_PARTIAL_CONTENT;
 #ifdef ENABLE_DEBUG
                         if (rangeVal) {
-                            std::cerr << "actual range header: " << range->HeaderValue() << "\n";
+                            server->context.debug->Info(TAG, str::Format("range header: %s", range->HeaderValue().c_str()).c_str());
                         }
 #endif
                     }
